

%% File description:
% This is the main file to run a bootstrap mean-variance portfolio as
% originally proposed by Michaud (1998). The baseline argument is the
% classical mean-variance portfolio proposed by Markowitz (1952). The bootstrap 
% methodology is detailed in Efron (1972).

%% References:
% Markowitz, H., Portfolio Selection, Journal of Finance (1952)
% Efron, B., Bootstrap methods: another look at the jackknife, Annals of
% Statistics (1972)
% Maddala, G.S. and Li, H., Bootstrap based tests in financial models,
% Handbook of statistics. Statistical methods in finance. Elsevier

%% Functions used
% ResamplingFrontier  = computes the resampled efficient frontier
% confregion   = computes the confidence regions
% stateqregion = computed the statistical equivalent mean variance portfolios 
% resampstat   = resampling portfolio statistics
% PlotFrontier = Plots the portfolio composition

%% TO BE RUN after Main_chap4 %%


%% Step 1. Construct the relevant time series of returns 
RetSeries = [GER.LTBond.ExRet(ss:se) GER.Stock.ExRet(ss:se) UK.Stock.ExRet(ss:se) US.Stock.ExRet(ss:se)];

%% Step 2. Compute the Resampled Frontier

NumPortf = 20;
NumSimu =  1000;

[Wrsp,ERrsp,SDrsp,Wmv,MuMv,StdMv,Wmv_S] = resampfront(RetSeries, NumPortf, NumSimu);

% Note:
% The outputs in the resampfront are the following:
% Wrsp  =  NumAssets time NumPortf matrix collecting the resampled portfolio weights
% ERrsp =  Column vector collecting resampled portfolio expected returns
% SRrsp =  Column vector collecting resampled portfolio standard deviations
% Wmv   =  Matrix collecting the mean-variance weights
% ERmv  =  Column vector collecting mean-variance portfolio expected returns
% SDmv  =  Column vector collectgin mean-variance portfolio standard deviations
% Wmv_S =  Array collecting the simulated mean-variance weights for statistical analysis purposes


    %% Step 3. Plots the confidence levels and the portfolios composition
    
PortfSet=[1,5,20]; % The set of portfolios considered
ConfLevel=5; % The confidence level 
confregion(Wrsp,ERrsp,SDrsp,MuMv,StdMv,Wmv_S,RetSeries,PortfSet,ConfLevel) % Plot the confidence region of the portfolios

% Plot the portfolio composition for each risk propensity

PlotFrontier_BT(Wmv','Mean-Variance Portfolios')   % Plot the Mean-Variance portfolios
PlotFrontier_BT(Wrsp','Mean-Variance Resampled Portfolios') % Plot the resampled Mean-Variance portfolios


%% Step 4. Plots the statistical equivalence region 

stateqregion(RetSeries, MuMv, StdMv, Wmv_S) 

%% Step 5. Generates the output statistics 

PortfSet1=[1:20];
[Stats] = Statistics(Wmv, Wmv_S, Wrsp, PortfSet1, ConfLevel); % Collect the simulations statistics

tElapsed = toc(tStart);
display(['The time needed to run the code is: ',num2str(tElapsed/60),' minutes']);                                                            







